import 'package:device_info_plus/device_info_plus.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter_vasat/models/vasat_model.dart';
import 'package:package_info_plus/package_info_plus.dart';

class Device extends VasatModel<Device> {

  String? osName;
  String? deviceType;
  String? make;
  String? model;
  String? osVersion;
  String? remoteId;
  double? x;
  double? y;
  double? height;
  String? crs;
  String? lastAppVersionCode;
  String? lastAppVersionName;
  String? pushId;
  bool allowPush = false;
  Map<String,dynamic>? meta;


  Device({Map<String, dynamic>? json}) {
    fromJson(json);
    super.fromJson(json);
  }

  factory Device.fromJson(Map<String, dynamic> json) {
    return Device(json:json);
  }

  static Future<Device> mkDevice() async {

    DeviceInfoPlugin deviceInfo = DeviceInfoPlugin();
    PackageInfo packageInfo = await PackageInfo.fromPlatform();
    Device device = Device();
    device.lastAppVersionName = packageInfo.version;
    device.lastAppVersionCode = packageInfo.buildNumber;

    // Check if web app
    if (kIsWeb) {
      WebBrowserInfo info = await deviceInfo.webBrowserInfo;
      device.deviceType = "Web";
      device.displayName = info.product;
      //device.remoteId = info.platform;
      device.osName = info.browserName.toString();
      device.make = info.vendor;
      device.model = info.vendorSub;
      device.osVersion = info.userAgent;

    } else if (defaultTargetPlatform == TargetPlatform.android ) {
      AndroidDeviceInfo info = await deviceInfo.androidInfo;
      device.deviceType = "Android";
      device.displayName = info.fingerprint;
      device.remoteId = info.androidId;
      device.osName = "Android - ${info.version.baseOS} ${info.version.codename}";
      device.osVersion = "${info.version.codename} ${info.version.sdkInt} ${info.version.securityPatch}";
      device.make = info.manufacturer;
      device.model = info.model;

    } else if (defaultTargetPlatform == TargetPlatform.iOS ) {
      IosDeviceInfo info = await deviceInfo.iosInfo;
      device.deviceType = "iOS";
      device.displayName = info.name;
      device.remoteId = info.identifierForVendor;
      device.osName = info.localizedModel;
      device.osVersion = info.systemVersion;
      device.make = info.systemName;
      device.model = info.model;

    } else if (defaultTargetPlatform == TargetPlatform.windows) {
      WindowsDeviceInfo info = await deviceInfo.windowsInfo;
      device.deviceType = "Windows";
      device.displayName = info.computerName;
      device.osName = "Windows";

    } else if (defaultTargetPlatform == TargetPlatform.linux) {
      LinuxDeviceInfo info = await deviceInfo.linuxInfo;
      device.deviceType = "Linux";
      device.displayName = info.name;
      device.remoteId = info.machineId;
      device.osName = info.prettyName;
      device.osVersion = info.version;
      device.make = info.variant;
      device.model = info.buildId;

    } else if (defaultTargetPlatform == TargetPlatform.macOS) {
      MacOsDeviceInfo info = await deviceInfo.macOsInfo;
      device.deviceType = "MacOS";
      device.displayName = info.computerName;
      device.osName = info.osRelease;
      device.osVersion = info.kernelVersion;
      device.make = info.hostName;
      device.model = info.model;
    } else if (defaultTargetPlatform == TargetPlatform.fuchsia) {
      device.deviceType = "Fuchsia";
      device.displayName = deviceInfo.toString();
    } else {
      device.deviceType = "Unknown";
      device.displayName = "Unknown";
    }
    return device;

  }

  @override
  fromJsonItem(Map<String, dynamic>? json) {
    if (json != null) {
      displayName = json["displayName"];
      osName = json['osName'];
      deviceType = json['deviceType'];
      make = json['make'];
      model = json['model'];
      osVersion = json['osVersion'];
      remoteId = json['remoteId'];
      if (json['lastLocation'] != null) {
        x = json['lastLocation']['x'];
        y = json['lastLocation']['y'];
        height = json['lastLocation']['height'];
        crs = json['lastLocation']['crs'];
      }
      lastAppVersionCode = json['lastAppVersionCode'];
      lastAppVersionName = json['lastAppVersionName'];
      pushId = json['pushId'];
      allowPush = json['allowPush'];
      meta = json['meta'];
    }
  }


  @override
  Map<String, dynamic> toJson() {
    var obj = <String, dynamic>{};
    obj["displayName"] = displayName;
    obj["osName"] = osName;
    obj["deviceType"] = deviceType;
    obj["make"] = make;
    obj["model"] = model;
    obj["osVersion"] = osVersion;
    obj["remoteId"] = remoteId;
    if (x != null && y != null) {
      obj["lastLocation"] = {
        "x": x,
        "y": y
      };
    }
    obj["lastAppVersionCode"] = lastAppVersionCode;
    obj["lastAppVersionName"] = lastAppVersionName;
    obj["pushId"] = pushId;
    obj["allowPush"] = allowPush;
    obj["meta"] = meta;
    return obj;
  }

  @override
  Device copyFromJson(Map<String, dynamic> json) {
    return Device.fromJson(json);
  }


}